(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 13.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     66305,       1123]
NotebookOptionsPosition[     63871,       1078]
NotebookOutlinePosition[     64293,       1095]
CellTagsIndexPosition[     64250,       1092]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 StyleBox["Peta4",
  FontColor->RGBColor[0.5, 0, 0.5]],
 " job notebook"
}], "Title",
 CellChangeTimes->{{3.856165344530184*^9, 3.856165360586437*^9}, {
   3.856592043342741*^9, 3.8565920465891027`*^9}, {3.8609516255848217`*^9, 
   3.860951628400779*^9}, 
   3.860958933575226*^9},ExpressionUUID->"77b3805d-060b-4070-8508-\
41576cdf0004"],

Cell["\<\
This notebook generates a Wolfram language file to be run on Peta4, a CPU \
cluster and component of the Cambridge Service for Data-Driven Discovery \
(CSD3).\
\>", "Text",
 CellChangeTimes->{{3.85659205646922*^9, 3.8565920652907*^9}, {
  3.860951668400402*^9, 3.8609516702246943`*^9}, {3.860959143061717*^9, 
  3.860959184437006*^9}},ExpressionUUID->"ac3b952a-323b-405e-9ac1-\
f072cdc53b32"],

Cell["\<\
We use the $Timing environment to activate some (undocumented) functionality \
in HiGGS, which clocks function calls and allows us to plot the HPC \
electroencephalograph (EEG). Beware, this is a very inefficient and \
data-intensive process!\
\>", "Text",
 CellChangeTimes->{{3.861995247671852*^9, 
  3.861995355087325*^9}},ExpressionUUID->"794dafc1-e87b-4d0f-b4e9-\
0b5888c33eda"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Timing", "=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860694729913267*^9, 3.8606947345521383`*^9}, {
  3.860941328673038*^9, 3.860941330424848*^9}, {3.8610183117224417`*^9, 
  3.861018312626536*^9}},ExpressionUUID->"0598b0f1-b4ed-48e5-84a9-\
d6000881fded"],

Cell["\<\
There are two options for the peta4 job. Firstly we can run the \
\[OpenCurlyDoubleQuote]Commissioning Survey\[CloseCurlyDoubleQuote], i.e. \
calculate all the brackets for the batches of theories created in \
peta4.svy.m. Careful analysis of this survey indicated that there were two \
cases of interest when targeting strong coupling. For these cases, we would \
additionally like to know about the velocities: this is a tough calculation, \
poorly parallelized, so if we go for this option it is better to modify \
peta4.job.slm to an array of e.g. two nodes.\
\>", "Text",
 CellChangeTimes->{{3.8619954058339863`*^9, 3.861995409548527*^9}, {
  3.8619954590701027`*^9, 
  3.861995734483922*^9}},ExpressionUUID->"3b5519f5-12e0-475d-ab82-\
bb8c58f5cbeb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CommissioningSurvey", "=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.861995745476265*^9, 3.8619957590353327`*^9}, {
  3.862038948480648*^9, 
  3.862038950409245*^9}},ExpressionUUID->"5f3f8005-173c-4c7f-8821-\
f61eba7d3105"],

Cell["\<\
If we choose to run the survey, we need to import a batch of jobs based on \
the node number allocated by the SLURM array index variable. Sometimes we are \
doing some calibration on the login node, in which case we pass peta4.job.m \
no command line option for any node, and the batch of jobs is simply taken to \
be the Yo-Nester theories without any geometric multipliers. \
\>", "Text",
 CellChangeTimes->{{3.8619958039300737`*^9, 3.861995824602295*^9}, {
  3.861995877417856*^9, 
  3.861996002713149*^9}},ExpressionUUID->"e9e38e54-7d82-4f89-85cf-\
43e689908a7a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"IntegerQ", "@", 
     RowBox[{"ToExpression", "@", 
      RowBox[{"$CommandLine", "[", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"$Node", "=", 
      RowBox[{"$CommandLine", "[", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ToExpression", "@", 
      RowBox[{"(", 
       RowBox[{"\"\<<<\>\"", "<>", 
        RowBox[{"FileNameJoin", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Directory", "[", "]"}], ",", "\"\<svy\>\"", ",", 
           RowBox[{"\"\<node-\>\"", "<>", "$Node"}], ",", 
           "\"\<peta4.svy.mx\>\""}], "}"}]}], "<>", "\"\<;\>\""}], ")"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "@", "JobsBatch"}], ";"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"$Node", "=", "\"\<\>\""}], ";", "\[IndentingNewLine]", 
     RowBox[{"JobsBatch", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<spin_0p\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp1", "==", "0"}], ",", 
            RowBox[{"Alp2", "==", "0"}], ",", 
            RowBox[{"Alp3", "==", "0"}], ",", 
            RowBox[{"Alp4", "==", "0"}], ",", 
            RowBox[{"Alp5", "==", "0"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"2", "Bet1"}], "+", "Bet2"}], "==", "0"}], ",", 
            RowBox[{
             RowBox[{"Bet1", "+", 
              RowBox[{"2", "Bet3"}]}], "==", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<spin_0m\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp1", "==", "0"}], ",", 
            RowBox[{"Alp2", "==", "0"}], ",", 
            RowBox[{"Alp4", "==", "0"}], ",", 
            RowBox[{"Alp5", "==", "0"}], ",", 
            RowBox[{"Alp6", "==", "0"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"2", "Bet1"}], "+", "Bet2"}], "==", "0"}], ",", 
            RowBox[{
             RowBox[{"Bet1", "+", 
              RowBox[{"2", "Bet3"}]}], "==", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<simple_spin_1p\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet1", "==", "0"}], ",", 
            RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<simple_spin_1m\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet1", "==", "0"}], ",", 
            RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\"\<simple_spin_2m\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet1", "==", "0"}], ",", 
            RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\"\<simple_spin_0-2m_a\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet1", "==", "0"}], ",", 
            RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<simple_spin_0-2m_b\>\"", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
            RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet1", "==", "0"}], ",", 
            RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
            RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp1", "==", "0"}], ",", 
            RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
            RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
            RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}]}], "}"}]}], 
     ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8603905757908*^9, 3.8603905771856337`*^9}, {
   3.8603906366890163`*^9, 3.860390646321033*^9}, {3.86039135532191*^9, 
   3.860391358729986*^9}, {3.8603922173772907`*^9, 3.860392236692824*^9}, {
   3.860392322544469*^9, 3.860392323264037*^9}, {3.8603924496419888`*^9, 
   3.860392470310701*^9}, {3.8603925008149*^9, 3.860392522158063*^9}, 
   3.860392574511548*^9, {3.860392715140317*^9, 3.860392728020187*^9}, {
   3.8603927901801033`*^9, 3.860392815214188*^9}, {3.860392858918503*^9, 
   3.86039286716334*^9}, {3.860393219815696*^9, 3.860393237902969*^9}, {
   3.8603932754949427`*^9, 3.8603932872068233`*^9}, {3.860393640715701*^9, 
   3.860393671867011*^9}, {3.860394010071816*^9, 3.86039401023975*^9}, {
   3.860397922128271*^9, 3.860397923400187*^9}, {3.860398256701365*^9, 
   3.860398258069018*^9}, {3.860398522946157*^9, 3.860398643385234*^9}, {
   3.860398693177548*^9, 3.8603987004433317`*^9}, {3.860398794359468*^9, 
   3.8603988214708242`*^9}, {3.860403937499442*^9, 3.86040395073501*^9}, {
   3.860406473346457*^9, 3.860406473538062*^9}, {3.8604296855961323`*^9, 
   3.8604297002273407`*^9}, {3.860429737922781*^9, 3.860429785250402*^9}, {
   3.860429817181283*^9, 3.860429895866373*^9}, 3.8609605278069077`*^9, {
   3.860963114351927*^9, 
   3.8609631149117327`*^9}},ExpressionUUID->"a60e8bc9-0d86-412a-8600-\
18c7b30e6c81"],

Cell["The Peta4 architecture equates cores to processors.", "Text",
 CellChangeTimes->{{3.860964186202428*^9, 
  3.8609642122661858`*^9}},ExpressionUUID->"ee52d6a2-e148-4542-baaa-\
4943d57775fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Remove", " ", "artificial", " ", "single"}], "-", 
    RowBox[{
    "core", " ", "machine", " ", "limit", " ", "for", " ", "cluster"}]}], 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Unprotect", "[", "$ProcessorCount", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
    "Skylake", " ", "nodes", " ", "have", " ", "32", " ", "processors", " ", 
     "apice"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"$ProcessorCount", "=", "32"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
     "Due", " ", "to", " ", "issues", " ", "contacting", " ", "the", " ", 
      "license", " ", "server"}], ",", " ", 
     RowBox[{
     "it", " ", "is", " ", "necessary", " ", "to", " ", "force", " ", 
      "this"}]}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"$Cores", "=", "32"}], ";"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, 3.860044767183367*^9, {3.8600501884979267`*^9, 
   3.860050190728992*^9}, {3.860050222695786*^9, 3.860050240304451*^9}, {
   3.860051976431161*^9, 3.860051976698119*^9}, {3.86007842241339*^9, 
   3.860078422741357*^9}, {3.86007958297482*^9, 3.860079589892458*^9}, {
   3.860254795985964*^9, 3.860254796273282*^9}, {3.8603905581247187`*^9, 
   3.86039056046061*^9}, {3.860392391857956*^9, 3.860392410433288*^9}, {
   3.860392737542636*^9, 3.860392740334879*^9}, {3.860393553164215*^9, 
   3.860393556699789*^9}, {3.86101831564247*^9, 3.861018353474094*^9}, {
   3.8624216766096992`*^9, 3.86242167783294*^9}, {3.8624227811658373`*^9, 
   3.862422784566442*^9}, {3.862425187637657*^9, 3.862425193549714*^9}, {
   3.862426211627945*^9, 3.86242622011528*^9}, {3.862427289400815*^9, 
   3.862427292792797*^9}, {3.862428184960107*^9, 3.862428190072304*^9}, {
   3.862430005029182*^9, 3.862430009533592*^9}, {3.862431069379809*^9, 
   3.862431073107834*^9}, {3.8624634463736267`*^9, 
   3.862463449693465*^9}},ExpressionUUID->"ecfae8f8-9928-4519-ae15-\
3f06f02b8c48"],

Cell["Load and build HiGGS.", "Text",
 CellChangeTimes->{{3.86199601461628*^9, 
  3.86199602037624*^9}},ExpressionUUID->"8cadafe2-ebaa-4368-8892-\
872c76afd29d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"Load", " ", "HiGGS"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Needs", "[", "\"\<xAct`HiGGS`\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"Build", " ", "the", " ", "HiGGS", " ", "environment"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"BuildHiGGS", "[", "]"}], ";"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, 3.860044767183367*^9, {3.8600501884979267`*^9, 
   3.860050190728992*^9}, {3.860050222695786*^9, 3.860050240304451*^9}, {
   3.860051976431161*^9, 3.860051976698119*^9}, {3.86007842241339*^9, 
   3.860078422741357*^9}, {3.86007958297482*^9, 3.860079589892458*^9}, {
   3.860254795985964*^9, 3.860254796273282*^9}, {3.8603905581247187`*^9, 
   3.86039056046061*^9}, {3.860392391857956*^9, 3.860392410433288*^9}, {
   3.860392737542636*^9, 3.860392743422007*^9}, {3.8606946431208677`*^9, 
   3.860694644272841*^9}},ExpressionUUID->"cae13836-2cd3-4564-b2d1-\
8cd274f6bb3c"],

Cell["\<\
This is where we do the job. Remember that either we are doing \
\[OpenCurlyDoubleQuote]Commissioning Survey\[CloseCurlyDoubleQuote] \
(including calibration), so we want to process a batch, or we are targeting a \
pair of cases later on, in which case we want to find also the velocities.\
\>", "Text",
 CellChangeTimes->{{3.861996252077731*^9, 
  3.861996323637498*^9}},ExpressionUUID->"394febfe-10f8-4bde-bc1a-\
ad0ae8c7010c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{"CommissioningSurvey", ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"StudyTheory", "[", 
      RowBox[{"JobsBatch", ",", 
       RowBox[{"\"\<Import\>\"", "\[Rule]", "False"}], ",", 
       RowBox[{"\"\<Velocities\>\"", "\[Rule]", "False"}]}], "]"}], ";"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"StudyTheory", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\"\<simple_spin_1p_A3A4A6B1B3_velocities\>\"", "<>", 
            RowBox[{"ToString", "@", "$Node"}]}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
             RowBox[{"Bet1", "==", "0"}], ",", 
             RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp1", "==", "0"}], ",", 
             RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
             RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
             RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\"\<simple_spin_1p_A3A6B1B2B3_velocities\>\"", "<>", 
            RowBox[{"ToString", "@", "$Node"}]}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
             RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
             RowBox[{"Bet1", "==", "0"}], ",", 
             RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp1", "==", "0"}], ",", 
             RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
             RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
             RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
             RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
             RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}]}], "}"}], 
       ",", 
       RowBox[{"\"\<Import\>\"", "\[Rule]", "False"}], ",", 
       RowBox[{"\"\<Velocities\>\"", "\[Rule]", "True"}]}], "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, 3.860044767183367*^9, {3.8600501884979267`*^9, 
   3.860050190728992*^9}, {3.860050222695786*^9, 3.860050240304451*^9}, {
   3.860051976431161*^9, 3.860051976698119*^9}, {3.86007842241339*^9, 
   3.860078422741357*^9}, {3.86007958297482*^9, 3.860079589892458*^9}, {
   3.860254795985964*^9, 3.860254796273282*^9}, {3.8603905581247187`*^9, 
   3.86039056046061*^9}, {3.860392391857956*^9, 3.860392410433288*^9}, {
   3.860392737542636*^9, 3.860392743422007*^9}, 3.8604039471978207`*^9, {
   3.8604064768576727`*^9, 3.8604064815304117`*^9}, {3.8610769522448997`*^9, 
   3.8610769880049667`*^9}, {3.861684268542276*^9, 3.861684299185758*^9}, {
   3.8616844026187353`*^9, 3.86168441029397*^9}, {3.86168444939684*^9, 
   3.861684503511599*^9}, {3.861684621835538*^9, 3.861684631306999*^9}, {
   3.8616855043251534`*^9, 3.8616855080554247`*^9}, {3.861796606893694*^9, 
   3.861796643653099*^9}, 3.861870625205268*^9, {3.861995765452095*^9, 
   3.8619957878139753`*^9}, {3.8619960255847263`*^9, 
   3.861996125639751*^9}},ExpressionUUID->"e2c58913-e1e8-4976-9444-\
297f506ffb31"],

Cell["If we get this far, we can go back to peta4.job.sh.", "Text",
 CellChangeTimes->{{3.861996195182335*^9, 
  3.861996209702506*^9}},ExpressionUUID->"7a732aea-e398-47d6-b27a-\
6360f447923e"],

Cell[BoxData[
 RowBox[{"Print", "[", 
  "\"\<calls to HiGGS functions have finished, reached the end of peta4.job.m\
\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860409681587123*^9, 3.86040970536922*^9}, {
  3.860413075335601*^9, 3.860413087071505*^9}, {3.860476773175561*^9, 
  3.860476794582974*^9}, {3.860964353697081*^9, 
  3.86096435471316*^9}},ExpressionUUID->"53af60b6-775f-495b-84cc-\
c241bc4146a0"],

Cell["\<\
Close parallel kernels to give the right exit code to SLURM, you mustn\
\[CloseCurlyQuote]t let the kernels loiter.\
\>", "Text",
 CellChangeTimes->{{3.860964365538021*^9, 3.860964384568491*^9}, {
  3.8619961426628637`*^9, 
  3.861996152247435*^9}},ExpressionUUID->"636218a9-69ad-46db-aff0-\
325a7f81a551"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CloseKernels", "[", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860409681587123*^9, 3.86040970536922*^9}, {
  3.860413075335601*^9, 3.860413087071505*^9}, {3.860476773175561*^9, 
  3.860476794582974*^9}, {3.860964353697081*^9, 
  3.860964386856573*^9}},ExpressionUUID->"e4217b10-d107-4106-b268-\
77b2f1425036"],

Cell["Also for the right exit code, master kernel suicide.", "Text",
 CellChangeTimes->{{3.861996158696113*^9, 
  3.861996187038691*^9}},ExpressionUUID->"441d5cc0-f5e3-46b1-8152-\
f3dc81831c40"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Quit", "[", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, 3.860044767183367*^9, {3.8600501884979267`*^9, 
   3.860050190728992*^9}, {3.860050222695786*^9, 3.860050240304451*^9}, {
   3.860051976431161*^9, 3.860051976698119*^9}, {3.86007842241339*^9, 
   3.860078422741357*^9}, {3.86007958297482*^9, 3.860079589892458*^9}, {
   3.860254795985964*^9, 3.860254796273282*^9}, {3.8603905581247187`*^9, 
   3.86039056046061*^9}, {3.860392391857956*^9, 3.860392410433288*^9}, {
   3.860392737542636*^9, 3.860392743422007*^9}, 3.8604039471978207`*^9, {
   3.8604064768576727`*^9, 
   3.8604064815304117`*^9}},ExpressionUUID->"e62bc551-0b92-4abb-81fc-\
3bafa72a1f11"]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1920, 1147},
WindowMargins->{{Automatic, 1080}, {Automatic, 53}},
TaggingRules->Association["TryRealOnly" -> False],
FrontEndVersion->"11.3 for Linux x86 (64-bit) (March 6, 2018)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 354, 9, 99, "Title",ExpressionUUID->"77b3805d-060b-4070-8508-41576cdf0004"],
Cell[937, 33, 402, 8, 36, "Text",ExpressionUUID->"ac3b952a-323b-405e-9ac1-f072cdc53b32"],
Cell[1342, 43, 392, 8, 36, "Text",ExpressionUUID->"794dafc1-e87b-4d0f-b4e9-0b5888c33eda"],
Cell[1737, 53, 328, 7, 47, "Input",ExpressionUUID->"0598b0f1-b4ed-48e5-84a9-d6000881fded",
 InitializationCell->True],
Cell[2068, 62, 765, 13, 60, "Text",ExpressionUUID->"3b5519f5-12e0-475d-ab82-bb8c58f5cbeb"],
Cell[2836, 77, 292, 7, 47, "Input",ExpressionUUID->"5f3f8005-173c-4c7f-8821-f61eba7d3105",
 InitializationCell->True],
Cell[3131, 86, 577, 10, 60, "Text",ExpressionUUID->"e9e38e54-7d82-4f89-85cf-43e689908a7a"],
Cell[3711, 98, 9790, 205, 324, "Input",ExpressionUUID->"a60e8bc9-0d86-412a-8600-18c7b30e6c81",
 InitializationCell->True],
Cell[13504, 305, 195, 3, 36, "Text",ExpressionUUID->"ee52d6a2-e148-4542-baaa-4943d57775fa"],
Cell[13702, 310, 12152, 181, 163, "Input",ExpressionUUID->"ecfae8f8-9928-4519-ae15-3f06f02b8c48",
 InitializationCell->True],
Cell[25857, 493, 161, 3, 36, "Text",ExpressionUUID->"8cadafe2-ebaa-4368-8892-872c76afd29d"],
Cell[26021, 498, 11113, 159, 119, "Input",ExpressionUUID->"cae13836-2cd3-4564-b2d1-8cd274f6bb3c",
 InitializationCell->True],
Cell[37137, 659, 439, 8, 36, "Text",ExpressionUUID->"394febfe-10f8-4bde-bc1a-ad0ae8c7010c"],
Cell[37579, 669, 13948, 214, 186, "Input",ExpressionUUID->"e2c58913-e1e8-4976-9444-297f506ffb31",
 InitializationCell->True],
Cell[51530, 885, 193, 3, 36, "Text",ExpressionUUID->"7a732aea-e398-47d6-b27a-6360f447923e"],
Cell[51726, 890, 435, 9, 47, "Input",ExpressionUUID->"53af60b6-775f-495b-84cc-c241bc4146a0",
 InitializationCell->True],
Cell[52164, 901, 316, 7, 36, "Text",ExpressionUUID->"636218a9-69ad-46db-aff0-325a7f81a551"],
Cell[52483, 910, 374, 8, 47, "Input",ExpressionUUID->"e4217b10-d107-4106-b268-77b2f1425036",
 InitializationCell->True],
Cell[52860, 920, 194, 3, 36, "Text",ExpressionUUID->"441d5cc0-f5e3-46b1-8152-f3dc81831c40"],
Cell[53057, 925, 10798, 150, 47, "Input",ExpressionUUID->"e62bc551-0b92-4abb-81fc-3bafa72a1f11",
 InitializationCell->True]
}, Open  ]]
}
]
*)

